 /***************************************************************************\
 *                                                                           *
 *                            Multiple Cogs Demo                             *
 *                                                                           *
 * This program shows how to use the cog function _coginit_C to start a      *
 * many cogs running C functions                                             *
 *                                                                           *
 \***************************************************************************/

#include <propeller.h>

#define STACK_SIZE 10

#define MAX_COGS 8

/*
 * blink_function : C function that can be executed in a cog.
 *                 (the only requirement for such a function is that it 
 *                 be a void function that requires no parameters - to 
 *                 share data with it, use commmon variables)
 */
void blink_function(void) {
   int me = _cogid();
   int pin = me + 56;
   int on_off = 0;

   while (1) {
      on_off ^= 1;
      DIRB |= 1<<(pin - 32);
      OUTB ^= on_off<<(pin-32);
      _waitcnt ((pin)<<16);
   }
}

/*
 * main : start a blink function in all free cogs
 */
int main(void) {
   int cog = 0;
   unsigned long stacks[STACK_SIZE * MAX_COGS];

   // start up to MAX_COGS instances of blink_function 
   // (i.e. until there are no cogs left)
   for (cog = 0; cog < MAX_COGS; cog++) {
      _coginit_C(&blink_function, &stacks[STACK_SIZE*(cog+1)]);
   }

   // now blink ourselves
   blink_function();
   return 0;
}
